//=============================================================================
// Compatibility Save&Load.js
//=============================================================================
/*:
 * @plugindesc Makes changes to the Improved Save & Load Menu mod if it exists
 *
 * @author Pyro#3607
 *
 * @help
 *
 * Makes changes to the Improved Save & Load Menu mod if it exists
 *
 * TERMS OF USE
 * This plugin is licensed under the "Do What the Fuck You Want To Public License". Use it how you see fit. Do credit me if you want, though.
 *
 * COMPATIBILITY
 * There shouldn't be any issues. Load after the original plugin. Will do nothing if you don't have the mod installed.
 */
(function() {

  //=============================================================================
  // * Change Save Face Image
  //=============================================================================

  Game_Actor.prototype.faceSaveLoad = function() {
    //THIS WILL APPEAR IF  IS NOT INSTALLED/ENABLED
    return "01_FA_OMORI_BATTLE"
  };
  
  Game_Actor.prototype.faceSaveLoad2 = function() {
    var actor = this.actor();
    // When changing these the .png should not be required.
    switch (actor.id) {
      case 1: // Omori
      return "01_OMORI_BATTLE";
      case 2: // Aubrey
      return "02_AUBREY_BATTLE";
      case 3: // Kel
      return "03_KEL_BATTLE";
      case 4: // Hero
      return "04_HERO_BATTLE";
      case 5: // Basil
      return "05_BASIL_BATTLE";
      case 8: // Omori
      return "01_FA_OMORI_BATTLE";
      case 9: // Aubrey
      return "02_FA_AUBREY_BATTLE";
      case 10: // Kel
      return "03_FA_KEL_BATTLE";
      case 11: // Hero
      return "04_FA_HERO_BATTLE";
      default:
        return "01_OMORI_BATTLE";
    }
  };

  //=============================================================================
  // * Change Save Face Index
  //=============================================================================
Game_Actor.prototype.faceSaveLoadIndex = function() {
    var actor = this.actor();
    // When changing these the .png should not be required.
    switch (actor.id) {
      case 1: // Omori
      return 0;
      case 2: // Aubrey
      return 0;
      case 3: // Kel
      return 0;
      case 4: // Hero
      return 0;
      default:
        return 0;
    }
  };

  //=============================================================================
  // * Change Save Background
  //============================================================================= 
  hasMod = false;
  if (typeof Scene_OmoriFile.prototype.createCommandHints=== "function") { 
    hasMod = true;
  }
  if (hasMod) 
  {
    Window_OmoriFileInformation.prototype.refresh = function(valid, info, id) {
      console.log("refresh mod")
      // Clear Contents
      this.contents.clear();
      // Get Color
      var color = 'rgba(255, 255, 255, 1)';
      // Get ID
      //var id = this._index + 1;
      //var valid = DataManager.isThisGameFile(id);
      //var info = DataManager.loadSavefileInfo(id);
    
      // Draw Lines
      this.contents.fillRect(0, 29, this.contents.width, 3, color);
    
      for (var i = 0; i < 3; i++) {
        var y = 55 + (i * 25)
        this.contents.fillRect(113, y, this.contents.width - 117, 1, color);
      };
    
    
      // Draw File
      this.contents.fontSize = 30;
      if(this._columns === 1)this.contents.drawText('FILE ' + id + ':', 10 + 30, -5, 100, this.contents.fontSize);
      else{
        if(!valid || info.saveName === null || info.saveName === "" ||info.saveName === undefined)this.contents.drawText(id, 2, -5, 100, this.contents.fontSize, 'center');
        else this.contents.drawText(info.saveName, 2, -5, 98, this.contents.fontSize, 'center');
      }
      
      // If Valid
      if (valid) {
        if(this._select){
          this._faceSprite.visible = true;
          this._selectedBG.visible = true;
        }else{
          this._faceSprite.visible = false;
          this._selectedBG.visible = false;
        }
                    this.contents.drawText(info.dummyChapter, 85 + 30 + 13*Math.floor(Math.log10(id)), -5, this.contents.width, this.contents.fontSize);
      
          this.contents.fontSize = 28;
      
          let backBitmap = ImageManager.loadSystem('loadscreen_backgrounds');
          let width = 100;
          let height = 100;
          // this.contents.blt(backBitmap, 0, 0, width, height, 0, 34, width + 10, height);
          bgLocation = getBackground(info.location);
          this.contents.blt(backBitmap, width*bgLocation[0], height*bgLocation[1], width, height, 1, 33); //width*n, height*m controls background
          this._selectedBG.setFrame(width*bgLocation[0], height*bgLocation[1], width, height);
          // Get Actor
          var actor = info.actorData;
          // Draw Actor Face
          let faceSpriteData = actor.dummyFaceName ? actor.dummyFaceName : actor.faceName
          //console.log(actor);
          this.drawFace(faceSpriteData, 0, -2, this.contents.height - Window_Base._faceHeight + 7, Window_Base._faceWidth, height - 2);
          this._faceSprite.actor = actor;
          if(omoDelete)this._faceSprite.setAnimRow(4);
          else this._faceSprite.setAnimRow(0);
          // Draw Actor Name
          this.contents.fontSize = 24;
          if(info.saveName === null || info.saveName === "" || info.saveName === undefined)this.contents.drawText(actor.name, 118, 30, 200, 24);
          else this.contents.drawText(info.saveName, 118, 30, 200, 24);
          // Draw Level
          this.contents.drawText('LEVEL:', 290 + 55, 30, 100, 24);
          this.contents.drawText(actor.level, 290 + 55, 30, 70, 24, 'right');
          // Draw Total PlayTime
          this.contents.drawText('TOTAL PLAYTIME:', 118, 55, 200, 24);
          this.contents.drawText(info.playtime, 295 + 55, 55, 100, 24);
          // Draw Location
          this.contents.drawText('LOCATION:', 118, 80, 200, 24);
          this.contents.drawText(info.location, 205, 80, 210, 24, 'right');
        }else{
          this._faceSprite.visible = false;
          this._selectedBG.visible = false;
        };
    
      // Draw Border
      this.contents.fillRect(102, 32, 3, 102, 'rgba(255, 255, 255, 1)')
      this.contents.fillRect(0, 29, 108, 3, 'rgba(255, 255, 255, 1)')
    };
  } else 
  {
    Window_OmoriFileInformation.prototype.refresh = function() {
      console.log("refresh vanilla")
      // Clear Contents
      this.contents.clear();
      // Get Color
      var color = 'rgba(255, 255, 255, 1)';
      // Get ID
      var id = this._index + 1;
      var valid = DataManager.isThisGameFile(id);
      var info = DataManager.loadSavefileInfo(id);
    
      // Draw Lines
      this.contents.fillRect(0, 29, this.contents.width, 3, color);
      for (var i = 0; i < 3; i++) {
        var y = 55 + (i * 25)
        this.contents.fillRect(113, y, this.contents.width - 117, 1, color);
      };
    
    
      // Draw File
      this.contents.fontSize = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").refresh_contents_fontsize;
      let loc_position = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").file_position
      this.contents.drawText(LanguageManager.getMessageData("XX_BLUE.Omori_Save_Load").file.format(id), loc_position[0], loc_position[1], 100, this.contents.fontSize);
      // If Valid
      if (valid) {
        loc_position = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").refresh_drawText_position;
        let chap = LanguageManager.getMessageData("XX_BLUE.Chapter_Names")[info.chapter]
        if(!chap) {
          chap = info.chapter
        }
        this.contents.drawText(chap, loc_position[0], loc_position[1], this.contents.width, this.contents.fontSize);
        this.contents.fontSize = 28;
    
        let backBitmap = ImageManager.loadSystem('faceset_states');
        let width = backBitmap.width / 4;
        let height = backBitmap.height / 5;
        // this.contents.blt(backBitmap, 0, 0, width, height, 0, 34, width + 10, height);
        this.contents.blt(backBitmap, 0, 0, width, height, 1, 33);
        // Get Actor
        var actor = info.actorData
        // Draw Actor Face
        let bit = ImageManager.loadFace(actor.faceName);
        bit.addLoadListener(() => this.drawFace(actor.faceName, actor.faceIndex, -2, this.contents.height - Window_Base._faceHeight + 7, Window_Base._faceWidth, height - 2));
        // Draw Actor Name
        this.contents.fontSize = 24;
        this.contents.drawText(actor.name, 118, 30, 100, 24);
        // Draw Level
        loc_position = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").level_position;
        this.contents.drawText(LanguageManager.getMessageData("XX_BLUE.Omori_Save_Load").level, loc_position[0], loc_position[1], 100, 24);
        this.contents.drawText(actor.level, loc_position[0], loc_position[1], 70, 24, 'right');
        // Draw Total PlayTime
        loc_position = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").playtime_position;
        this.contents.drawText(LanguageManager.getMessageData("XX_BLUE.Omori_Save_Load").playtime, 118, 55, 200, 24);
        this.contents.drawText(info.playtime, loc_position[0], loc_position[1], 100, 24);
        // Draw Location
        this.contents.drawText(LanguageManager.getMessageData("XX_BLUE.Omori_Save_Load").location, 118, 80, 200, 24);
        this.contents.drawText(info.location, 205, 80, 210, 24, 'right');
      };
    }
  }


DataManager.makeSavefileInfo = function() {
// Get Original Info
var info = _TDS_.OmoriSaveLoad.DataManager_makeSavefileInfo.call(this);
// Get Leader
var actor = $gameParty.leader();
// "DUMMY SAVE FILE" only appears when the mod is not installed/enabled
info.actorData = {name: "DUMMY SAVE FILE", dummyName: actor.name(), level: actor.level, faceName: actor.faceSaveLoad(), faceIndex: actor.faceSaveLoadIndex(), dummyFaceName: actor.faceSaveLoad2()};
info.chapter = "DUMMY SAVE FILE"
info.dummyChapter = $gameVariables.value(23);
info.location = $gameMap.displayName();
info.saveName = $gameSystem.saveName;
// Return Info
return info;
};

Window_OmoriFileStats.prototype.updateStats = function (valid, info, id) {
this.contents.clear();
this.contents.fontSize = 30;
this.contents.drawText('FILE ' + id + ':', 1, 1, 130, this.contents.fontSize, 'center');
if (valid) {
  this.contents.drawText(info.dummyChapter, 1, 31, 130, this.contents.fontSize, 'center');
  var actor = info.actorData;
  // Draw Actor Name
  this.contents.fontSize = 24;
  // Draw Level
  this.contents.drawText('LEVEL: '+actor.level, 1, 67, 130, 24, 'center');
  // Draw Total PlayTime
  this.contents.drawText('PLAYTIME:', 1, 93, 130, 24, 'center');
  this.contents.drawText(info.playtime, 1, 117, 130, 24, 'center');
  // Draw Location
  this.contents.drawText('LOCATION:', 1, 143, 130, 24, 'center');
  this.contents.drawText(info.location, 1, 167, 130, 24, 'center');
}
var color = 'rgba(255, 255, 255, 1)';
this.contents.fillRect(0, 64, this.contents.width, 3, color);
this.contents.fillRect(8, 92, this.contents.width - 16, 1, color);
this.contents.fillRect(8, 142, this.contents.width - 16, 1, color);
this.contents.fillRect(8, 191, this.contents.width - 16, 1, color);
};

Sprite_OmoSaveMenuFace.prototype.updateBitmap = function() {
// Get Actor
var actor = this.actor
// If Actor Exists and it has Battle Status Face Name
if (actor) {
  let faceName = actor.dummyFaceName ? actor.dummyFaceName : actor.faceName
  // Set Bitmap
  this.bitmap = ImageManager.loadFace(faceName);
} else {
  this.bitmap = null;
};
// Update Frame
this.updateFrame();
};

function getBackground(location){
  switch(location){
    case "MAP NAME HERE":
      return [0,0];
    default:
      return [0,0];
  }
}

//=============================================================================
// * RESERVE IMAGE FACES
//=============================================================================
let old_loadReservedBitmaps = Scene_OmoriFile.prototype.loadReservedBitmaps;
Scene_OmoriFile.prototype.loadReservedBitmaps = function() {
  old_loadReservedBitmaps.call(this);
  // Super Call
  Scene_Base.prototype.loadReservedBitmaps.call(this);
  // Go through face
  ImageManager.reserveFace("01_OMORI_BATTLE", 0, this._imageReservationId);
  ImageManager.reserveFace("02_AUBREY_BATTLE", 0, this._imageReservationId);
  ImageManager.reserveFace("03_KEL_BATTLE", 0, this._imageReservationId);
  ImageManager.reserveFace("04_HERO_BATTLE", 0, this._imageReservationId);
  ImageManager.reserveFace("01_FA_OMORI_BATTLE", 0, this._imageReservationId);
  ImageManager.reserveFace("02_FA_AUBREY_BATTLE", 0, this._imageReservationId);
  ImageManager.reserveFace("03_FA_KEL_BATTLE", 0, this._imageReservationId);
  ImageManager.reserveFace("04_FA_HERO_BATTLE", 0, this._imageReservationId);

  ImageManager.reserveSystem('loadscreen_backgrounds', 0, this._imageReservationId);
  ImageManager.reserveBattleback1('battleback_bookshelf', 0, this._imageReservationId);
  ImageManager.reserveParallax('!parallax_black_space', 0, this._imageReservationId);
  ImageManager.reserveParallax('Space_parallax', 0, this._imageReservationId);
  ImageManager.reserveParallax('!polaroidBG_FA_day', 0, this._imageReservationId);
  ImageManager.reserveSystem('VISION',0,this._imageReservationId);
};

DataManager.saveGameWithoutRescue = function(savefileId) {
  // FOR SOME REASON THE GAME LIMITS THE SAVE DATA AND IT PROBABLY BREAKS STUFF
  var json = JsonEx.stringify(this.makeSaveContents());
  if (json.length >= 6000000) {
      console.warn('Save data too big!');
  }
  StorageManager.save(savefileId, json);
  this._lastAccessedId = savefileId;
  var globalInfo = this.loadGlobalInfo() || [];
  globalInfo[savefileId] = this.makeSavefileInfo();
  this.saveGlobalInfo(globalInfo);
  return true;
};

Sprite_OmoSaveMenuFace.prototype.updateBitmap = function() {
  // Get Actor
  var actor = this.actor
  // If Actor Exists and it has Battle Status Face Name
  if (actor) {
    let faceName = actor.dummyFaceName ? actor.dummyFaceName : actor.faceName
    // Set Bitmap
    this.bitmap = ImageManager.loadFace(faceName);
  } else {
    this.bitmap = null;
  };
  // Update Frame
  this.updateFrame();
}

})();
